@@ -50,10 +50,29 @@ pub fn gc_init<VM: VMBinding>(mmtk: &'static mut MMTK<VM>, heap_size: usize) {
             "MMTk failed to initialize the logger. Possibly a logger has been initialized by user."
         ),
     }
+    #[cfg(all(feature = "perf_counter", target_os = "linux"))]
+    {
+        use std::fs::File;
+        use std::io::Read;
+        let mut status = File::open("/proc/self/status").unwrap();
+        let mut contents = String::new();
+        status.read_to_string(&mut contents).unwrap();
+        for line in contents.lines() {
+            let split: Vec<&str> = line.split('\t').collect();
+            if split[0] == "Threads:" {
+                let threads = split[1].parse::<i32>().unwrap();
+                if threads != 1 {
+                    warn!("Current process has {} threads, process-wide perf event measurement will only include child threads spawned from this threadas", threads);
+                }
+            }
+        }
+    }
     assert!(heap_size > 0, "Invalid heap size");
     mmtk.plan
         .gc_init(heap_size, &crate::VM_MAP, &mmtk.scheduler);
     info!("Initialized MMTk with {:?}", mmtk.options.plan);
+    #[cfg(feature = "extreme_assertions")]
+    warn!("The feature 'extreme_assertions' is enabled. MMTk will run expensive run-time checks. Slow performance should be expected.");
 }
 
 /// Request MMTk to create a mutator for the given thread. For performance reasons, A VM should
@@ -414,3 +433,9 @@ pub fn add_work_packets<VM: VMBinding>(
 ) {
     mmtk.scheduler.work_buckets[bucket].bulk_add(packets)
 }
+
+/// Add a callback to be notified after the transitive closure is finished.
+/// The callback should return true if it add more work packets to the closure bucket.
+pub fn on_closure_end<VM: VMBinding>(mmtk: &'static MMTK<VM>, f: Box<dyn Send + Fn() -> bool>) {
+    mmtk.scheduler.on_closure_end(f)
+}
