@@ -48,7 +48,12 @@ pub const GENCOPY_CONSTRAINTS: PlanConstraints = PlanConstraints {
     gc_header_bits: 2,
     gc_header_words: 0,
     num_specialized_scans: 1,
+    needs_log_bit: true,
     barrier: super::ACTIVE_BARRIER,
+    max_non_los_default_alloc_bytes: crate::util::rust_util::min_of_usize(
+        crate::plan::plan_constraints::MAX_NON_LOS_ALLOC_BYTES_COPYING_PLAN,
+        crate::util::options::NURSERY_SIZE,
+    ),
     ..PlanConstraints::default()
 };
 
@@ -123,6 +128,13 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
         // Prepare global/collectors/mutators
         scheduler.work_buckets[WorkBucketStage::Prepare]
             .add(Prepare::<Self, GenCopyCopyContext<VM>>::new(self));
+        if is_full_heap {
+            scheduler.work_buckets[WorkBucketStage::RefClosure]
+                .add(ProcessWeakRefs::<GenCopyMatureProcessEdges<VM>>::new());
+        } else {
+            scheduler.work_buckets[WorkBucketStage::RefClosure]
+                .add(ProcessWeakRefs::<GenCopyNurseryProcessEdges<VM>>::new());
+        }
         // Release global/collectors/mutators
         scheduler.work_buckets[WorkBucketStage::Release]
             .add(Release::<Self, GenCopyCopyContext<VM>>::new(self));
@@ -138,9 +150,10 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
     }
 
     fn prepare(&mut self, tls: VMWorkerThread) {
-        self.common.prepare(tls, true);
+        let full_heap = !self.is_current_gc_nursery();
+        self.common.prepare(tls, full_heap);
         self.nursery.prepare(true);
-        if !self.is_current_gc_nursery() {
+        if full_heap {
             self.hi
                 .store(!self.hi.load(Ordering::SeqCst), Ordering::SeqCst); // flip the semi-spaces
         }
@@ -150,9 +163,10 @@ impl<VM: VMBinding> Plan for GenCopy<VM> {
     }
 
     fn release(&mut self, tls: VMWorkerThread) {
-        self.common.release(tls, true);
+        let full_heap = !self.is_current_gc_nursery();
+        self.common.release(tls, full_heap);
         self.nursery.release();
-        if !self.is_current_gc_nursery() {
+        if full_heap {
             self.fromspace().release();
         }
 
@@ -200,7 +214,7 @@ impl<VM: VMBinding> GenCopy<VM> {
     ) -> Self {
         let mut heap = HeapMeta::new(HEAP_START, HEAP_END);
         let gencopy_specs = if super::ACTIVE_BARRIER == BarrierSelector::ObjectBarrier {
-            metadata::extract_side_metadata(&[VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC])
+            metadata::extract_side_metadata(&[*VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC])
         } else {
             vec![]
         };
@@ -211,7 +225,7 @@ impl<VM: VMBinding> GenCopy<VM> {
                 "nursery",
                 false,
                 true,
-                VMRequest::fixed_extent(options.max_nursery, false),
+                VMRequest::fixed_extent(crate::util::options::NURSERY_SIZE, false),
                 global_metadata_specs.clone(),
                 vm_map,
                 mmapper,
@@ -250,6 +264,8 @@ impl<VM: VMBinding> GenCopy<VM> {
             next_gc_full_heap: AtomicBool::new(false),
         };
 
+        // Use SideMetadataSanity to check if each spec is valid. This is also needed for check
+        // side metadata in extreme_assertions.
         {
             let mut side_metadata_sanity_checker = SideMetadataSanity::new();
             res.common
