@@ -126,6 +126,7 @@ pub fn create_mutator<VM: VMBinding>(
         PlanSelector::MarkSweep => {
             crate::plan::marksweep::mutator::create_ms_mutator(tls, &*mmtk.plan)
         }
+        PlanSelector::Immix => crate::plan::immix::mutator::create_immix_mutator(tls, &*mmtk.plan),
         PlanSelector::PageProtect => {
             crate::plan::pageprotect::mutator::create_pp_mutator(tls, &*mmtk.plan)
         }
@@ -137,6 +138,7 @@ pub fn create_plan<VM: VMBinding>(
     vm_map: &'static VMMap,
     mmapper: &'static Mmapper,
     options: Arc<UnsafeOptionsWrapper>,
+    scheduler: Arc<MMTkScheduler<VM>>,
 ) -> Box<dyn Plan<VM = VM>> {
     match plan {
         PlanSelector::NoGC => Box::new(crate::plan::nogc::NoGC::new(vm_map, mmapper, options)),
@@ -149,6 +151,9 @@ pub fn create_plan<VM: VMBinding>(
         PlanSelector::MarkSweep => Box::new(crate::plan::marksweep::MarkSweep::new(
             vm_map, mmapper, options,
         )),
+        PlanSelector::Immix => Box::new(crate::plan::immix::Immix::new(
+            vm_map, mmapper, options, scheduler,
+        )),
         PlanSelector::PageProtect => Box::new(crate::plan::pageprotect::PageProtect::new(
             vm_map, mmapper, options,
         )),
@@ -160,6 +165,17 @@ pub fn create_plan<VM: VMBinding>(
 ///
 /// The global instance defines and manages static resources
 /// (such as memory and virtual memory resources).
+///
+/// Constructor:
+///
+/// For the constructor of a new plan, there are a few things the constructor _must_ do
+/// (please check existing plans and see what they do in the constructor):
+/// 1. Create a HeapMeta, and use this HeapMeta to initialize all the spaces.
+/// 2. Create a vector of all the side metadata specs with `SideMetadataContext::new_global_specs()`,
+///    the parameter is a vector of global side metadata specs that are specific to the plan.
+/// 3. Initialize all the spaces the plan uses with the heap meta, and the global metadata specs vector.
+/// 4. Create a `SideMetadataSanity` object, and invoke verify_side_metadata_sanity() for each space (or
+///    invoke verify_side_metadata_sanity() in `CommonPlan`/`BasePlan` for the spaces in the common/base plan).
 pub trait Plan: 'static + Sync + Downcast {
     type VM: VMBinding;
 
@@ -293,7 +309,7 @@ pub trait Plan: 'static + Sync + Downcast {
 
     fn handle_user_collection_request(&self, tls: VMMutatorThread, force: bool) {
         if force || !self.options().ignore_system_g_c {
-            info!("User triggerring collection");
+            info!("User triggering collection");
             self.base()
                 .user_triggered_collection
                 .store(true, Ordering::Relaxed);
@@ -364,6 +380,8 @@ pub struct BasePlan<VM: VMBinding> {
     // Spaces in base plan
     #[cfg(feature = "code_space")]
     pub code_space: ImmortalSpace<VM>,
+    #[cfg(feature = "code_space")]
+    pub code_lo_space: ImmortalSpace<VM>,
     #[cfg(feature = "ro_space")]
     pub ro_space: ImmortalSpace<VM>,
     #[cfg(feature = "vm_space")]
@@ -411,7 +429,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         constraints: &'static PlanConstraints,
         global_side_metadata_specs: Vec<SideMetadataSpec>,
     ) -> BasePlan<VM> {
-        let stats = Stats::new();
+        let stats = Stats::new(&options);
         // Initializing the analysis manager and routines
         #[cfg(feature = "analysis")]
         let analysis_manager = AnalysisManager::new(&stats);
@@ -427,6 +445,17 @@ impl<VM: VMBinding> BasePlan<VM> {
                 &mut heap,
                 constraints,
             ),
+            #[cfg(feature = "code_space")]
+            code_lo_space: ImmortalSpace::new(
+                "code_lo_space",
+                true,
+                VMRequest::discontiguous(),
+                global_side_metadata_specs.clone(),
+                vm_map,
+                mmapper,
+                &mut heap,
+                constraints,
+            ),
             #[cfg(feature = "ro_space")]
             ro_space: ImmortalSpace::new(
                 "ro_space",
@@ -491,6 +520,8 @@ impl<VM: VMBinding> BasePlan<VM> {
 
         #[cfg(feature = "code_space")]
         self.code_space.init(vm_map);
+        #[cfg(feature = "code_space")]
+        self.code_lo_space.init(vm_map);
         #[cfg(feature = "ro_space")]
         self.ro_space.init(vm_map);
         #[cfg(feature = "vm_space")]
@@ -509,6 +540,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         #[cfg(feature = "code_space")]
         {
             pages += self.code_space.reserved_pages();
+            pages += self.code_lo_space.reserved_pages();
         }
         #[cfg(feature = "ro_space")]
         {
@@ -531,6 +563,12 @@ impl<VM: VMBinding> BasePlan<VM> {
             return self.code_space.trace_object::<T>(_trace, _object);
         }
 
+        #[cfg(feature = "code_space")]
+        if self.code_lo_space.in_space(_object) {
+            trace!("trace_object: object in large code space");
+            return self.code_lo_space.trace_object::<T>(_trace, _object);
+        }
+
         #[cfg(feature = "ro_space")]
         if self.ro_space.in_space(_object) {
             trace!("trace_object: object in ro_space space");
@@ -548,6 +586,8 @@ impl<VM: VMBinding> BasePlan<VM> {
     pub fn prepare(&mut self, _tls: VMWorkerThread, _primary: bool) {
         #[cfg(feature = "code_space")]
         self.code_space.prepare();
+        #[cfg(feature = "code_space")]
+        self.code_lo_space.prepare();
         #[cfg(feature = "ro_space")]
         self.ro_space.prepare();
         #[cfg(feature = "vm_space")]
@@ -557,6 +597,8 @@ impl<VM: VMBinding> BasePlan<VM> {
     pub fn release(&mut self, _tls: VMWorkerThread, _primary: bool) {
         #[cfg(feature = "code_space")]
         self.code_space.release();
+        #[cfg(feature = "code_space")]
+        self.code_lo_space.release();
         #[cfg(feature = "ro_space")]
         self.ro_space.release();
         #[cfg(feature = "vm_space")]
@@ -612,7 +654,7 @@ impl<VM: VMBinding> BasePlan<VM> {
         *self.gc_status.lock().unwrap() == GcStatus::GcProper
     }
 
-    fn is_user_triggered_collection(&self) -> bool {
+    pub fn is_user_triggered_collection(&self) -> bool {
         self.user_triggered_collection.load(Ordering::Relaxed)
     }
 
@@ -842,11 +884,12 @@ use enum_map::Enum;
 /// Allocation semantics that MMTk provides.
 /// Each allocation request requires a desired semantic for the object to allocate.
 #[repr(i32)]
-#[derive(Clone, Copy, Debug, Enum)]
+#[derive(Clone, Copy, Debug, Enum, PartialEq, Eq)]
 pub enum AllocationSemantics {
     Default = 0,
     Immortal = 1,
     Los = 2,
     Code = 3,
     ReadOnly = 4,
+    LargeCode = 5,
 }
