@@ -37,6 +37,16 @@ impl<VM: VMBinding> SFT for ImmortalSpace<VM> {
     fn is_live(&self, _object: ObjectReference) -> bool {
         true
     }
+    #[inline(always)]
+    fn is_reachable(&self, object: ObjectReference) -> bool {
+        let old_value = load_metadata::<VM>(
+            &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+            object,
+            None,
+            Some(Ordering::SeqCst),
+        );
+        old_value == self.mark_state
+    }
     fn is_movable(&self) -> bool {
         false
     }
@@ -46,19 +56,23 @@ impl<VM: VMBinding> SFT for ImmortalSpace<VM> {
     }
     fn initialize_object_metadata(&self, object: ObjectReference, _alloc: bool) {
         let old_value = load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+            &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
             object,
             None,
             Some(Ordering::SeqCst),
         );
         let new_value = (old_value & GC_MARK_BIT_MASK) | self.mark_state;
         store_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+            &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
             object,
             new_value,
             None,
             Some(Ordering::SeqCst),
         );
+
+        if self.common.needs_log_bit {
+            VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC.mark_as_unlogged::<VM>(object, Ordering::SeqCst);
+        }
     }
 }
 
@@ -94,19 +108,20 @@ impl<VM: VMBinding> ImmortalSpace<VM> {
         vm_map: &'static VMMap,
         mmapper: &'static Mmapper,
         heap: &mut HeapMeta,
-        _constraints: &'static PlanConstraints,
+        constraints: &'static PlanConstraints,
     ) -> Self {
         let common = CommonSpace::new(
             SpaceOptions {
                 name,
                 movable: false,
                 immortal: true,
+                needs_log_bit: constraints.needs_log_bit,
                 zeroed,
                 vmrequest,
                 side_metadata_specs: SideMetadataContext {
                     global: global_side_metadata_specs,
                     local: metadata::extract_side_metadata(&[
-                        VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+                        *VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
                     ]),
                 },
             },
@@ -133,7 +148,7 @@ impl<VM: VMBinding> ImmortalSpace<VM> {
     fn test_and_mark(object: ObjectReference, value: usize) -> bool {
         loop {
             let old_value = load_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+                &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
                 object,
                 None,
                 Some(Ordering::SeqCst),
@@ -143,7 +158,7 @@ impl<VM: VMBinding> ImmortalSpace<VM> {
             }
 
             if compare_exchange_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
+                &VM::VMObjectModel::LOCAL_MARK_BIT_SPEC,
                 object,
                 old_value,
                 old_value ^ GC_MARK_BIT_MASK,
