@@ -52,7 +52,7 @@ impl<VM: VMBinding> SFT for LargeObjectSpace<VM> {
     }
     fn initialize_object_metadata(&self, object: ObjectReference, alloc: bool) {
         let old_value = load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+            &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
             object,
             None,
             Some(Ordering::SeqCst),
@@ -62,13 +62,17 @@ impl<VM: VMBinding> SFT for LargeObjectSpace<VM> {
             new_value |= NURSERY_BIT;
         }
         store_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+            &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
             object,
             new_value,
             None,
             Some(Ordering::SeqCst),
         );
 
+        if self.common.needs_log_bit {
+            VM::VMObjectModel::GLOBAL_LOG_BIT_SPEC.mark_as_unlogged::<VM>(object, Ordering::SeqCst);
+        }
+
         let cell = VM::VMObjectModel::object_start_ref(object);
         self.treadmill.add_to_treadmill(cell, alloc);
     }
@@ -84,7 +88,9 @@ impl<VM: VMBinding> Space<VM> for LargeObjectSpace<VM> {
     fn get_page_resource(&self) -> &dyn PageResource<VM> {
         &self.pr
     }
-    fn init(&mut self, _vm_map: &'static VMMap) {}
+    fn init(&mut self, _vm_map: &'static VMMap) {
+        self.common().init(self.as_space());
+    }
 
     fn common(&self) -> &CommonSpace<VM> {
         &self.common
@@ -105,7 +111,7 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
         vm_map: &'static VMMap,
         mmapper: &'static Mmapper,
         heap: &mut HeapMeta,
-        _constraints: &'static PlanConstraints,
+        constraints: &'static PlanConstraints,
         protect_memory_on_release: bool,
     ) -> Self {
         let common = CommonSpace::new(
@@ -114,11 +120,12 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
                 movable: false,
                 immortal: false,
                 zeroed,
+                needs_log_bit: constraints.needs_log_bit,
                 vmrequest,
                 side_metadata_specs: SideMetadataContext {
                     global: global_side_metadata_specs,
                     local: metadata::extract_side_metadata(&[
-                        VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+                        *VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
                     ]),
                 },
             },
@@ -208,7 +215,7 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
                 MARK_BIT
             };
             let old_value = load_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+                &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
                 object,
                 None,
                 Some(Ordering::SeqCst),
@@ -218,7 +225,7 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
                 return false;
             }
             if compare_exchange_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+                &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
                 object,
                 old_value,
                 old_value & !LOS_BIT_MASK | value,
@@ -234,7 +241,7 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
 
     fn test_mark_bit(&self, object: ObjectReference, value: usize) -> bool {
         load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+            &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
             object,
             None,
             Some(Ordering::SeqCst),
@@ -245,7 +252,7 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
     /// Check if a given object is in nursery
     fn is_in_nursery(&self, object: ObjectReference) -> bool {
         load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+            &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
             object,
             None,
             Some(Ordering::Relaxed),
@@ -257,14 +264,14 @@ impl<VM: VMBinding> LargeObjectSpace<VM> {
     fn clear_nursery(&self, object: ObjectReference) {
         loop {
             let old_val = load_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+                &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
                 object,
                 None,
                 Some(Ordering::Relaxed),
             );
             let new_val = old_val & !NURSERY_BIT;
             if compare_exchange_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
+                &VM::VMObjectModel::LOCAL_LOS_MARK_NURSERY_SPEC,
                 object,
                 old_val,
                 new_val,
