@@ -49,8 +49,19 @@ use downcast_rs::Downcast;
 pub trait SFT {
     /// The space name
     fn name(&self) -> &str;
+    /// Get forwarding pointer if the object is forwarded.
+    #[inline(always)]
+    fn get_forwarded_object(&self, _object: ObjectReference) -> Option<ObjectReference> {
+        None
+    }
     /// Is the object live, determined by the policy?
     fn is_live(&self, object: ObjectReference) -> bool;
+    /// Is the object reachable, determined by the policy?
+    /// Note: Objects in ImmortalSpace may have `is_live = true` but are actually unreachable.
+    #[inline(always)]
+    fn is_reachable(&self, object: ObjectReference) -> bool {
+        self.is_live(object)
+    }
     /// Is the object movable, determined by the policy? E.g. the policy is non-moving,
     /// or the object is pinned.
     fn is_movable(&self) -> bool;
@@ -471,6 +482,10 @@ pub struct CommonSpace<VM: VMBinding> {
 
     pub metadata: SideMetadataContext,
 
+    /// This field equals to needs_log_bit in the plan constraints.
+    // TODO: This should be a constant for performance.
+    pub needs_log_bit: bool,
+
     p: PhantomData<VM>,
 }
 
@@ -479,6 +494,7 @@ pub struct SpaceOptions {
     pub movable: bool,
     pub immortal: bool,
     pub zeroed: bool,
+    pub needs_log_bit: bool,
     pub vmrequest: VMRequest,
     pub side_metadata_specs: SideMetadataContext,
 }
@@ -506,6 +522,7 @@ impl<VM: VMBinding> CommonSpace<VM> {
             head_discontiguous_region: unsafe { Address::zero() },
             vm_map,
             mmapper,
+            needs_log_bit: opt.needs_log_bit,
             metadata: opt.side_metadata_specs,
             p: PhantomData,
         };
