@@ -1,29 +1,26 @@
+use super::mmapper::MapState;
 use super::Mmapper;
+use crate::util::conversions;
 use crate::util::heap::layout::vm_layout_constants::*;
 use crate::util::Address;
-use crate::util::{conversions, side_metadata::SideMetadata};
 use atomic::{Atomic, Ordering};
 use std::fmt;
 use std::io::Result;
 use std::mem::transmute;
 use std::sync::Mutex;
 
-#[repr(u8)]
-#[derive(Copy, Clone, PartialEq, Eq, Debug)]
-enum MapState {
-    Unmapped,
-    Mapped,
-    Protected,
-}
-
 const MMAP_NUM_CHUNKS: usize = 1 << (33 - LOG_MMAP_CHUNK_BYTES);
 
-const LOG_MAPPABLE_BYTES: usize = 36; // 128GB - physical memory larger than this is uncommon
-                                      /*
-                                       * Size of a slab.  The value 10 gives a slab size of 1GB, with 1024
-                                       * chunks per slab, ie a 1k slab map.  In a 64-bit address space, this
-                                       * will require 1M of slab maps.
-                                       */
+// 36 = 128G - physical memory larger than this is uncommon
+// 40 = 2T. Increased to 2T. Though we probably won't use this much memory, we allow quarantine memory range,
+// and that is usually used to quarantine a large amount of memory.
+const LOG_MAPPABLE_BYTES: usize = 40;
+
+/*
+ * Size of a slab.  The value 10 gives a slab size of 1GB, with 1024
+ * chunks per slab, ie a 1k slab map.  In a 64-bit address space, this
+ * will require 1M of slab maps.
+ */
 const LOG_MMAP_CHUNKS_PER_SLAB: usize = 8;
 const LOG_MMAP_SLAB_BYTES: usize = LOG_MMAP_CHUNKS_PER_SLAB + LOG_MMAP_CHUNK_BYTES;
 const MMAP_SLAB_EXTENT: usize = 1 << LOG_MMAP_SLAB_BYTES;
@@ -83,12 +80,39 @@ impl Mmapper for FragmentedMapper {
         }
     }
 
-    fn ensure_mapped(
-        &self,
-        mut start: Address,
-        pages: usize,
-        metadata: &SideMetadata,
-    ) -> Result<()> {
+    fn quarantine_address_range(&self, mut start: Address, pages: usize) -> Result<()> {
+        let end = start + conversions::pages_to_bytes(pages);
+        // Iterate over the slabs covered
+        while start < end {
+            let base = Self::slab_align_down(start);
+            let high = if end > Self::slab_limit(start) && !Self::slab_limit(start).is_zero() {
+                Self::slab_limit(start)
+            } else {
+                end
+            };
+
+            let slab = Self::slab_align_down(start);
+            let start_chunk = Self::chunk_index(slab, start);
+            let end_chunk = Self::chunk_index(slab, conversions::mmap_chunk_align_up(high));
+
+            let mapped = self.get_or_allocate_slab_table(start);
+
+            /* Iterate over the chunks within the slab */
+            for (chunk, entry) in mapped.iter().enumerate().take(end_chunk).skip(start_chunk) {
+                if matches!(entry.load(Ordering::Relaxed), MapState::Quarantined) {
+                    continue;
+                }
+
+                let mmap_start = Self::chunk_index_to_address(base, chunk);
+                let _guard = self.lock.lock().unwrap();
+                MapState::transition_to_quarantined(entry, mmap_start).unwrap();
+            }
+            start = high;
+        }
+        Ok(())
+    }
+
+    fn ensure_mapped(&self, mut start: Address, pages: usize) -> Result<()> {
         let end = start + conversions::pages_to_bytes(pages);
         // Iterate over the slabs covered
         while start < end {
@@ -111,31 +135,12 @@ impl Mmapper for FragmentedMapper {
                     continue;
                 }
 
+                let mmap_start = Self::chunk_index_to_address(base, chunk);
                 let _guard = self.lock.lock().unwrap();
-                match entry.load(Ordering::Relaxed) {
-                    MapState::Unmapped => {
-                        let mmap_start = Self::chunk_index_to_address(base, chunk);
-                        // map data
-                        let res =
-                            crate::util::memory::dzmmap_noreplace(mmap_start, MMAP_CHUNK_BYTES);
-                        if res.is_err() {
-                            return res;
-                        }
-                        // map metadata
-                        let res = self.map_metadata(mmap_start, metadata);
-                        if res.is_err() {
-                            return res;
-                        }
-                    }
-                    MapState::Protected => {
-                        let mmap_start = Self::chunk_index_to_address(base, chunk);
-                        crate::util::memory::munprotect(mmap_start, MMAP_CHUNK_BYTES).unwrap();
-                    }
-                    // might have become MAPPED here
-                    MapState::Mapped => {}
+                let res = MapState::transition_to_mapped(entry, mmap_start);
+                if res.is_err() {
+                    return res;
                 }
-
-                entry.store(MapState::Mapped, Ordering::Relaxed);
             }
             start = high;
         }
@@ -178,13 +183,8 @@ impl Mmapper for FragmentedMapper {
             let mapped = self.get_or_allocate_slab_table(start);
 
             for (chunk, entry) in mapped.iter().enumerate().take(end_chunk).skip(start_chunk) {
-                if entry.load(Ordering::Relaxed) == MapState::Mapped {
-                    let mmap_start = Self::chunk_index_to_address(base, chunk);
-                    crate::util::memory::mprotect(mmap_start, MMAP_CHUNK_BYTES).unwrap();
-                    entry.store(MapState::Protected, Ordering::Relaxed);
-                } else {
-                    debug_assert!(entry.load(Ordering::Relaxed) == MapState::Protected);
-                }
+                let mmap_start = Self::chunk_index_to_address(base, chunk);
+                MapState::transition_to_protected(entry, mmap_start).unwrap();
             }
             start = high;
         }
@@ -339,7 +339,6 @@ mod tests {
     use crate::util::constants::LOG_BYTES_IN_PAGE;
     use crate::util::heap::layout::vm_layout_constants::MMAP_CHUNK_BYTES;
     use crate::util::memory;
-    use crate::util::side_metadata::{SideMetadata, SideMetadataContext};
     use crate::util::test_util::FRAGMENTED_MMAPPER_TEST_REGION;
     use crate::util::test_util::{serial_test, with_cleanup};
     use crate::util::{conversions, Address};
@@ -347,13 +346,6 @@ mod tests {
     const FIXED_ADDRESS: Address = FRAGMENTED_MMAPPER_TEST_REGION.start;
     const MAX_BYTES: usize = FRAGMENTED_MMAPPER_TEST_REGION.size;
 
-    lazy_static! {
-        static ref NO_METADATA: SideMetadata = SideMetadata::new(SideMetadataContext {
-            global: vec![],
-            local: vec![]
-        });
-    }
-
     fn pages_to_chunks_up(pages: usize) -> usize {
         conversions::raw_align_up(pages, MMAP_CHUNK_BYTES) / MMAP_CHUNK_BYTES
     }
@@ -396,9 +388,7 @@ mod tests {
             with_cleanup(
                 || {
                     let mmapper = FragmentedMapper::new();
-                    mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages, &NO_METADATA)
-                        .unwrap();
+                    mmapper.ensure_mapped(FIXED_ADDRESS, pages).unwrap();
 
                     let chunks = pages_to_chunks_up(pages);
                     for i in 0..chunks {
@@ -424,9 +414,7 @@ mod tests {
             with_cleanup(
                 || {
                     let mmapper = FragmentedMapper::new();
-                    mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages, &NO_METADATA)
-                        .unwrap();
+                    mmapper.ensure_mapped(FIXED_ADDRESS, pages).unwrap();
 
                     let chunks = pages_to_chunks_up(pages);
                     for i in 0..chunks {
@@ -453,9 +441,7 @@ mod tests {
             with_cleanup(
                 || {
                     let mmapper = FragmentedMapper::new();
-                    mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages, &NO_METADATA)
-                        .unwrap();
+                    mmapper.ensure_mapped(FIXED_ADDRESS, pages).unwrap();
 
                     let chunks = pages_to_chunks_up(pages);
                     for i in 0..chunks {
@@ -484,7 +470,7 @@ mod tests {
                     let mmapper = FragmentedMapper::new();
                     let pages_per_chunk = MMAP_CHUNK_BYTES >> LOG_BYTES_IN_PAGE as usize;
                     mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2, &NO_METADATA)
+                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2)
                         .unwrap();
 
                     // protect 1 chunk
@@ -515,7 +501,7 @@ mod tests {
                     let mmapper = FragmentedMapper::new();
                     let pages_per_chunk = MMAP_CHUNK_BYTES >> LOG_BYTES_IN_PAGE as usize;
                     mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2, &NO_METADATA)
+                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2)
                         .unwrap();
 
                     // protect 1 chunk
@@ -532,7 +518,7 @@ mod tests {
 
                     // ensure mapped - this will unprotect the previously protected chunk
                     mmapper
-                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2, &NO_METADATA)
+                        .ensure_mapped(FIXED_ADDRESS, pages_per_chunk * 2)
                         .unwrap();
                     assert_eq!(
                         get_chunk_map_state(&mmapper, FIXED_ADDRESS),
