@@ -1,5 +1,5 @@
 use super::*;
-use crate::util::constants::BYTES_IN_PAGE;
+use crate::util::constants::{BYTES_IN_PAGE, LOG_BITS_IN_BYTE};
 use crate::util::heap::layout::vm_layout_constants::BYTES_IN_CHUNK;
 use crate::util::memory;
 use crate::util::{constants, Address};
@@ -16,27 +16,125 @@ use std::sync::atomic::{AtomicU16, AtomicU32, AtomicU8, AtomicUsize, Ordering};
 #[derive(Clone, Copy, PartialEq, Eq, Hash)]
 pub struct SideMetadataSpec {
     pub is_global: bool,
-    pub offset: usize,
+    pub offset: SideMetadataOffset,
     /// Number of bits needed per region. E.g. 0 = 1 bit, 1 = 2 bit.
     pub log_num_of_bits: usize,
     /// Number of bytes of the region. E.g. 3 = 8 bytes, 12 = 4096 bytes (page).
     pub log_min_obj_size: usize,
 }
 
+impl SideMetadataSpec {
+    /// Is offset for this spec Address? (contiguous side metadata for 64 bits, and global specs in 32 bits)
+    #[inline(always)]
+    pub const fn is_absolute_offset(&self) -> bool {
+        self.is_global || cfg!(target_pointer_width = "64")
+    }
+    /// If offset for this spec relative? (chunked side metadata for local specs in 32 bits)
+    #[inline(always)]
+    pub const fn is_rel_offset(&self) -> bool {
+        !self.is_absolute_offset()
+    }
+
+    #[inline(always)]
+    pub const fn get_absolute_offset(&self) -> Address {
+        debug_assert!(self.is_absolute_offset());
+        unsafe { self.offset.addr }
+    }
+
+    #[inline(always)]
+    pub const fn get_rel_offset(&self) -> usize {
+        debug_assert!(self.is_rel_offset());
+        unsafe { self.offset.rel_offset }
+    }
+}
+
 impl fmt::Debug for SideMetadataSpec {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.write_fmt(format_args!(
             "SideMetadataSpec {{ \
             **is_global: {:?} \
-            **offset: 0x{:x} \
+            **offset: {} \
             **log_num_of_bits: 0x{:x} \
             **log_min_obj_size: 0x{:x} \
             }}",
-            self.is_global, self.offset, self.log_num_of_bits, self.log_min_obj_size
+            self.is_global,
+            unsafe {
+                if self.is_absolute_offset() {
+                    format!("0x{:x}", self.offset.addr)
+                } else {
+                    format!("0x{:x}", self.offset.rel_offset)
+                }
+            },
+            self.log_num_of_bits,
+            self.log_min_obj_size
         ))
     }
 }
 
+/// A union of Address or relative offset (usize) used to store offset for a side metadata spec.
+/// If a spec is contiguous side metadata, it uses address. Othrewise it uses usize.
+// The fields are made private on purpose. They can only be accessed from SideMetadata which knows whether it is Address or usize.
+#[derive(Clone, Copy)]
+pub union SideMetadataOffset {
+    addr: Address,
+    rel_offset: usize,
+}
+
+impl SideMetadataOffset {
+    // Get an offset for a fixed address. This is usually used to set offset for the first spec (subsequent ones can be laid out with `layout_after`).
+    pub const fn addr(addr: Address) -> Self {
+        SideMetadataOffset { addr }
+    }
+
+    // Get an offset for a relative offset (usize). This is usually used to set offset for the first spec (subsequent ones can be laid out with `layout_after`).
+    pub const fn rel(rel_offset: usize) -> Self {
+        SideMetadataOffset { rel_offset }
+    }
+
+    /// Get an offset after a spec. This is used to layout another spec immediately after this one.
+    #[cfg(target_pointer_width = "64")]
+    pub const fn layout_after(spec: &SideMetadataSpec) -> SideMetadataOffset {
+        debug_assert!(spec.is_absolute_offset());
+        SideMetadataOffset {
+            addr: unsafe { spec.offset.addr }
+                .add(crate::util::metadata::side_metadata::metadata_address_range_size(spec)),
+        }
+    }
+    /// Get an offset after a spec. This is used to layout another spec immediately after this one.
+    #[cfg(target_pointer_width = "32")]
+    pub const fn layout_after(spec: &SideMetadataSpec) -> SideMetadataOffset {
+        if spec.is_absolute_offset() {
+            SideMetadataOffset {
+                addr: unsafe { spec.offset.addr }
+                    .add(crate::util::metadata::side_metadata::metadata_address_range_size(spec)),
+            }
+        } else {
+            SideMetadataOffset {
+                rel_offset: unsafe { spec.offset.rel_offset }
+                    + crate::util::metadata::side_metadata::metadata_bytes_per_chunk(
+                        spec.log_min_obj_size,
+                        spec.log_num_of_bits,
+                    ),
+            }
+        }
+    }
+}
+
+// Address and usize has the same layout, so we use usize for implementing these traits.
+
+impl PartialEq for SideMetadataOffset {
+    fn eq(&self, other: &Self) -> bool {
+        unsafe { self.rel_offset == other.rel_offset }
+    }
+}
+impl Eq for SideMetadataOffset {}
+
+impl std::hash::Hash for SideMetadataOffset {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        unsafe { self.rel_offset }.hash(state);
+    }
+}
+
 /// This struct stores all the side metadata specs for a policy. Generally a policy needs to know its own
 /// side metadata spec as well as the plan's specs.
 pub struct SideMetadataContext {
@@ -205,7 +303,7 @@ impl SideMetadataContext {
 
 // Used only for debugging
 // Panics in the required metadata for data_addr is not mapped
-pub fn ensure_metadata_is_mapped(metadata_spec: SideMetadataSpec, data_addr: Address) {
+pub fn ensure_metadata_is_mapped(metadata_spec: &SideMetadataSpec, data_addr: Address) {
     let meta_start = address_to_meta_address(metadata_spec, data_addr).align_down(BYTES_IN_PAGE);
 
     debug!(
@@ -217,7 +315,7 @@ pub fn ensure_metadata_is_mapped(metadata_spec: SideMetadataSpec, data_addr: Add
 }
 
 #[inline(always)]
-pub fn load_atomic(metadata_spec: SideMetadataSpec, data_addr: Address, order: Ordering) -> usize {
+pub fn load_atomic(metadata_spec: &SideMetadataSpec, data_addr: Address, order: Ordering) -> usize {
     #[cfg(feature = "extreme_assertions")]
     let _lock = sanity::SANITY_LOCK.lock().unwrap();
 
@@ -255,7 +353,7 @@ pub fn load_atomic(metadata_spec: SideMetadataSpec, data_addr: Address, order: O
 
 #[inline(always)]
 pub fn store_atomic(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     metadata: usize,
     order: Ordering,
@@ -306,7 +404,7 @@ pub fn store_atomic(
 
 #[inline(always)]
 pub fn compare_exchange_atomic(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     old_metadata: usize,
     new_metadata: usize,
@@ -408,7 +506,7 @@ pub fn compare_exchange_atomic(
 // same as Rust atomics, this wraps around on overflow
 #[inline(always)]
 pub fn fetch_add_atomic(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     val: usize,
     order: Ordering,
@@ -467,7 +565,7 @@ pub fn fetch_add_atomic(
 // same as Rust atomics, this wraps around on overflow
 #[inline(always)]
 pub fn fetch_sub_atomic(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     val: usize,
     order: Ordering,
@@ -533,7 +631,7 @@ pub fn fetch_sub_atomic(
 /// 2. Interleaving Non-atomic and atomic operations is undefined behaviour.
 ///
 #[inline(always)]
-pub unsafe fn load(metadata_spec: SideMetadataSpec, data_addr: Address) -> usize {
+pub unsafe fn load(metadata_spec: &SideMetadataSpec, data_addr: Address) -> usize {
     #[cfg(feature = "extreme_assertions")]
     let _lock = sanity::SANITY_LOCK.lock().unwrap();
 
@@ -580,7 +678,7 @@ pub unsafe fn load(metadata_spec: SideMetadataSpec, data_addr: Address) -> usize
 /// 2. Interleaving Non-atomic and atomic operations is undefined behaviour.
 ///
 #[inline(always)]
-pub unsafe fn store(metadata_spec: SideMetadataSpec, data_addr: Address, metadata: usize) {
+pub unsafe fn store(metadata_spec: &SideMetadataSpec, data_addr: Address, metadata: usize) {
     #[cfg(feature = "extreme_assertions")]
     let _lock = sanity::SANITY_LOCK.lock().unwrap();
 
@@ -618,6 +716,67 @@ pub unsafe fn store(metadata_spec: SideMetadataSpec, data_addr: Address, metadat
     sanity::verify_store(metadata_spec, data_addr, metadata);
 }
 
+/// A byte array in side-metadata
+pub struct MetadataByteArrayRef<const ENTRIES: usize> {
+    #[cfg(feature = "extreme_assertions")]
+    heap_range_start: Address,
+    #[cfg(feature = "extreme_assertions")]
+    spec: SideMetadataSpec,
+    data: &'static [u8; ENTRIES],
+}
+
+impl<const ENTRIES: usize> MetadataByteArrayRef<ENTRIES> {
+    /// Get a piece of metadata address range as a byte array.
+    ///
+    /// # Arguments
+    ///
+    /// * `metadata_spec` - The specification of the target side metadata.
+    /// * `start` - The starting address of the heap range.
+    /// * `bytes` - The size of the heap range.
+    ///
+    pub fn new(metadata_spec: &SideMetadataSpec, start: Address, bytes: usize) -> Self {
+        debug_assert_eq!(
+            metadata_spec.log_num_of_bits, LOG_BITS_IN_BYTE as usize,
+            "Each heap entry should map to a byte in side-metadata"
+        );
+        debug_assert_eq!(
+            bytes >> metadata_spec.log_min_obj_size,
+            ENTRIES,
+            "Heap range size and MetadataByteArray size does not match"
+        );
+        Self {
+            #[cfg(feature = "extreme_assertions")]
+            heap_range_start: start,
+            #[cfg(feature = "extreme_assertions")]
+            spec: *metadata_spec,
+            // # Safety
+            // The metadata memory is assumed to be mapped when accessing.
+            data: unsafe { &*address_to_meta_address(metadata_spec, start).to_ptr() },
+        }
+    }
+
+    /// Get the length of the array.
+    #[allow(clippy::len_without_is_empty)]
+    pub const fn len(&self) -> usize {
+        ENTRIES
+    }
+
+    /// Get a byte from the metadata byte array at the given index.
+    #[inline(always)]
+    #[allow(clippy::let_and_return)]
+    pub fn get(&self, index: usize) -> u8 {
+        #[cfg(feature = "extreme_assertions")]
+        let _lock = sanity::SANITY_LOCK.lock().unwrap();
+        let value = self.data[index];
+        #[cfg(feature = "extreme_assertions")]
+        {
+            let data_addr = self.heap_range_start + (index << self.spec.log_min_obj_size);
+            sanity::verify_load(&self.spec, data_addr, value as _);
+        }
+        value
+    }
+}
+
 /// Bulk-zero a specific metadata for a chunk.
 ///
 /// # Arguments
@@ -626,7 +785,7 @@ pub unsafe fn store(metadata_spec: SideMetadataSpec, data_addr: Address, metadat
 ///
 /// * `chunk_start` - The starting address of the chunk whose metadata is being zeroed.
 ///
-pub fn bzero_metadata(metadata_spec: SideMetadataSpec, start: Address, size: usize) {
+pub fn bzero_metadata(metadata_spec: &SideMetadataSpec, start: Address, size: usize) {
     #[cfg(feature = "extreme_assertions")]
     let _lock = sanity::SANITY_LOCK.lock().unwrap();
 
@@ -687,16 +846,18 @@ pub fn bzero_metadata(metadata_spec: SideMetadataSpec, start: Address, size: usi
 
 #[cfg(test)]
 mod tests {
+    use super::*;
     use crate::util::metadata::side_metadata::SideMetadataContext;
 
-    use super::*;
+    // offset is not used in these tests.
+    pub const ZERO_OFFSET: SideMetadataOffset = SideMetadataOffset { rel_offset: 0 };
 
     #[test]
     fn calculate_reserved_pages_one_spec() {
         // 1 bit per 8 bytes - 1:64
         let spec = SideMetadataSpec {
             is_global: true,
-            offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: ZERO_OFFSET,
             log_num_of_bits: 0,
             log_min_obj_size: 3,
         };
@@ -716,14 +877,14 @@ mod tests {
         // 1 bit per 8 bytes - 1:64
         let gspec = SideMetadataSpec {
             is_global: true,
-            offset: GLOBAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: ZERO_OFFSET,
             log_num_of_bits: 0,
             log_min_obj_size: 3,
         };
         // 2 bits per page - 2 / (4k * 8) = 1:16k
         let lspec = SideMetadataSpec {
             is_global: false,
-            offset: LOCAL_SIDE_METADATA_BASE_ADDRESS.as_usize(),
+            offset: ZERO_OFFSET,
             log_num_of_bits: 1,
             log_min_obj_size: 12,
         };
