@@ -134,10 +134,10 @@ fn verify_no_overlap_contiguous(
     spec_1: &SideMetadataSpec,
     spec_2: &SideMetadataSpec,
 ) -> Result<()> {
-    let end_1 = spec_1.offset + super::metadata_address_range_size(spec_1);
-    let end_2 = spec_2.offset + super::metadata_address_range_size(spec_2);
+    let end_1 = spec_1.get_absolute_offset() + super::metadata_address_range_size(spec_1);
+    let end_2 = spec_2.get_absolute_offset() + super::metadata_address_range_size(spec_2);
 
-    if !(spec_1.offset >= end_2 || spec_2.offset >= end_1) {
+    if !(spec_1.get_absolute_offset() >= end_2 || spec_2.get_absolute_offset() >= end_1) {
         return Err(Error::new(
             ErrorKind::InvalidInput,
             format!(
@@ -159,12 +159,12 @@ fn verify_no_overlap_contiguous(
 ///
 #[cfg(target_pointer_width = "32")]
 fn verify_no_overlap_chunked(spec_1: &SideMetadataSpec, spec_2: &SideMetadataSpec) -> Result<()> {
-    let end_1 = spec_1.offset
+    let end_1 = spec_1.get_rel_offset()
         + super::metadata_bytes_per_chunk(spec_1.log_min_obj_size, spec_1.log_num_of_bits);
-    let end_2 = spec_2.offset
+    let end_2 = spec_2.get_rel_offset()
         + super::metadata_bytes_per_chunk(spec_2.log_min_obj_size, spec_2.log_num_of_bits);
 
-    if !(spec_1.offset >= end_2 || spec_2.offset >= end_1) {
+    if !(spec_1.get_rel_offset() >= end_2 || spec_2.get_rel_offset() >= end_1) {
         return Err(Error::new(
             ErrorKind::InvalidInput,
             format!(
@@ -383,7 +383,7 @@ impl SideMetadataSanity {
 /// * `size`: size of the source data
 ///
 #[cfg(feature = "extreme_assertions")]
-pub fn verify_bzero(metadata_spec: SideMetadataSpec, start: Address, size: usize) {
+pub fn verify_bzero(metadata_spec: &SideMetadataSpec, start: Address, size: usize) {
     let sanity_map = &mut CONTENT_SANITY_MAP.write().unwrap();
     match sanity_map.get_mut(&metadata_spec) {
         Some(spec_sanity_map) => {
@@ -446,7 +446,7 @@ pub fn verify_load(metadata_spec: &SideMetadataSpec, data_addr: Address, actual_
 /// * `metadata`: the metadata content to store
 ///
 #[cfg(feature = "extreme_assertions")]
-pub fn verify_store(metadata_spec: SideMetadataSpec, data_addr: Address, metadata: usize) {
+pub fn verify_store(metadata_spec: &SideMetadataSpec, data_addr: Address, metadata: usize) {
     let sanity_map = &mut CONTENT_SANITY_MAP.write().unwrap();
     match sanity_map.get_mut(&metadata_spec) {
         Some(spec_sanity_map) => {
@@ -461,7 +461,7 @@ pub fn verify_store(metadata_spec: SideMetadataSpec, data_addr: Address, metadat
 /// A helper function encapsulating the common parts of addition and subtraction
 #[cfg(feature = "extreme_assertions")]
 fn do_math(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     val: usize,
     math_op: MathOp,
@@ -498,7 +498,7 @@ fn do_math(
 ///
 #[cfg(feature = "extreme_assertions")]
 pub fn verify_add(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     val_to_add: usize,
     actual_old_val: usize,
@@ -529,7 +529,7 @@ pub fn verify_add(
 ///
 #[cfg(feature = "extreme_assertions")]
 pub fn verify_sub(
-    metadata_spec: SideMetadataSpec,
+    metadata_spec: &SideMetadataSpec,
     data_addr: Address,
     val_to_sub: usize,
     actual_old_val: usize,
@@ -556,13 +556,13 @@ mod tests {
     fn test_side_metadata_sanity_verify_global_specs_total_size() {
         let spec_1 = SideMetadataSpec {
             is_global: true,
-            offset: 0,
+            offset: SideMetadataOffset::addr(Address::ZERO),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         let spec_2 = SideMetadataSpec {
             is_global: true,
-            offset: metadata_address_range_size(&spec_1),
+            offset: SideMetadataOffset::layout_after(&spec_1),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -575,7 +575,7 @@ mod tests {
 
         let spec_2 = SideMetadataSpec {
             is_global: true,
-            offset: metadata_address_range_size(&spec_1),
+            offset: SideMetadataOffset::layout_after(&spec_1),
             log_num_of_bits: 3,
             log_min_obj_size: 1,
         };
@@ -584,7 +584,7 @@ mod tests {
 
         let spec_1 = SideMetadataSpec {
             is_global: true,
-            offset: 0,
+            offset: SideMetadataOffset::addr(Address::ZERO),
             log_num_of_bits: 1,
             #[cfg(target_pointer_width = "64")]
             log_min_obj_size: 0,
@@ -593,7 +593,7 @@ mod tests {
         };
         let spec_2 = SideMetadataSpec {
             is_global: true,
-            offset: metadata_address_range_size(&spec_1),
+            offset: SideMetadataOffset::layout_after(&spec_1),
             log_num_of_bits: 3,
             #[cfg(target_pointer_width = "64")]
             log_min_obj_size: 2,
@@ -609,13 +609,13 @@ mod tests {
     fn test_side_metadata_sanity_verify_no_overlap_contiguous() {
         let spec_1 = SideMetadataSpec {
             is_global: true,
-            offset: 0,
+            offset: SideMetadataOffset::addr(Address::ZERO),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         let spec_2 = SideMetadataSpec {
             is_global: true,
-            offset: metadata_address_range_size(&spec_1),
+            offset: SideMetadataOffset::layout_after(&spec_1),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -625,7 +625,7 @@ mod tests {
 
         let spec_1 = SideMetadataSpec {
             is_global: true,
-            offset: 1,
+            offset: SideMetadataOffset::addr(unsafe { Address::from_usize(1) }),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -634,13 +634,16 @@ mod tests {
 
         let spec_1 = SideMetadataSpec {
             is_global: true,
-            offset: 0,
+            offset: SideMetadataOffset::addr(Address::ZERO),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         let spec_2 = SideMetadataSpec {
             is_global: true,
-            offset: (metadata_address_range_size(&spec_1) - 1),
+            // We specifically make up an invalid offset
+            offset: SideMetadataOffset::addr(
+                spec_1.get_absolute_offset() + metadata_address_range_size(&spec_1) - 1,
+            ),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -652,14 +655,14 @@ mod tests {
     #[test]
     fn test_side_metadata_sanity_verify_no_overlap_chunked() {
         let spec_1 = SideMetadataSpec {
-            is_global: true,
-            offset: 0,
+            is_global: false,
+            offset: SideMetadataOffset::rel(0),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         let spec_2 = SideMetadataSpec {
-            is_global: true,
-            offset: metadata_bytes_per_chunk(spec_1.log_min_obj_size, spec_1.log_num_of_bits),
+            is_global: false,
+            offset: SideMetadataOffset::layout_after(&spec_1),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -668,8 +671,8 @@ mod tests {
         assert!(verify_no_overlap_chunked(&spec_1, &spec_2).is_ok());
 
         let spec_1 = SideMetadataSpec {
-            is_global: true,
-            offset: 1,
+            is_global: false,
+            offset: SideMetadataOffset::rel(1),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -677,14 +680,19 @@ mod tests {
         assert!(verify_no_overlap_chunked(&spec_1, &spec_2).is_err());
 
         let spec_1 = SideMetadataSpec {
-            is_global: true,
-            offset: 0,
+            is_global: false,
+            offset: SideMetadataOffset::rel(0),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
         let spec_2 = SideMetadataSpec {
-            is_global: true,
-            offset: (metadata_bytes_per_chunk(spec_1.log_min_obj_size, spec_1.log_num_of_bits) - 1),
+            is_global: false,
+            // We make up an invalid offset
+            offset: SideMetadataOffset::rel(
+                spec_1.get_rel_offset()
+                    + metadata_bytes_per_chunk(spec_1.log_min_obj_size, spec_1.log_num_of_bits)
+                    - 1,
+            ),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
@@ -697,7 +705,7 @@ mod tests {
     fn test_side_metadata_sanity_verify_local_specs_size() {
         let spec_1 = SideMetadataSpec {
             is_global: false,
-            offset: 0,
+            offset: SideMetadataOffset::rel(0),
             log_num_of_bits: 0,
             log_min_obj_size: 0,
         };
