@@ -31,14 +31,14 @@ const FORWARDING_POINTER_MASK: usize = 0xffff_fffc;
 pub fn attempt_to_forward<VM: VMBinding>(object: ObjectReference) -> usize {
     loop {
         let old_value = load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+            &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
             object,
             None,
             Some(Ordering::SeqCst),
         );
         if old_value != FORWARDING_NOT_TRIGGERED_YET
             || compare_exchange_metadata::<VM>(
-                VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+                &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
                 object,
                 old_value,
                 old_value | BEING_FORWARDED,
@@ -68,7 +68,7 @@ pub fn spin_and_get_forwarded_object<VM: VMBinding>(
     let mut forwarding_bits = forwarding_bits;
     while forwarding_bits == BEING_FORWARDED {
         forwarding_bits = load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+            &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
             object,
             None,
             Some(Ordering::SeqCst),
@@ -95,7 +95,7 @@ pub fn forward_object<VM: VMBinding, CC: CopyContext>(
     let new_object = VM::VMObjectModel::copy(object, semantics, copy_context);
     if let Some(shift) = forwarding_bits_offset_in_forwarding_pointer::<VM>() {
         store_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
+            &VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
             object,
             new_object.to_address().as_usize() | (FORWARDED << shift),
             None,
@@ -104,7 +104,7 @@ pub fn forward_object<VM: VMBinding, CC: CopyContext>(
     } else {
         write_forwarding_pointer::<VM>(object, new_object);
         store_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+            &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
             object,
             FORWARDED,
             None,
@@ -116,7 +116,7 @@ pub fn forward_object<VM: VMBinding, CC: CopyContext>(
 
 pub fn is_forwarded<VM: VMBinding>(object: ObjectReference) -> bool {
     load_metadata::<VM>(
-        VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+        &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
         object,
         None,
         Some(Ordering::SeqCst),
@@ -125,7 +125,7 @@ pub fn is_forwarded<VM: VMBinding>(object: ObjectReference) -> bool {
 
 pub fn is_forwarded_or_being_forwarded<VM: VMBinding>(object: ObjectReference) -> bool {
     load_metadata::<VM>(
-        VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+        &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
         object,
         None,
         Some(Ordering::SeqCst),
@@ -144,7 +144,7 @@ pub fn state_is_being_forwarded(forwarding_bits: usize) -> bool {
 /// This function is used on new objects.
 pub fn clear_forwarding_bits<VM: VMBinding>(object: ObjectReference) {
     store_metadata::<VM>(
-        VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+        &VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
         object,
         0,
         None,
@@ -157,7 +157,7 @@ pub fn clear_forwarding_bits<VM: VMBinding>(object: ObjectReference) {
 pub fn read_forwarding_pointer<VM: VMBinding>(object: ObjectReference) -> ObjectReference {
     unsafe {
         Address::from_usize(load_metadata::<VM>(
-            VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
+            &VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
             object,
             Some(FORWARDING_POINTER_MASK),
             Some(Ordering::SeqCst),
@@ -174,7 +174,7 @@ pub fn write_forwarding_pointer<VM: VMBinding>(
 ) {
     trace!("GCForwardingWord::write({:#?}, {:x})\n", object, new_object);
     store_metadata::<VM>(
-        VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
+        &VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
         object,
         new_object.to_address().as_usize(),
         Some(FORWARDING_POINTER_MASK),
@@ -191,10 +191,11 @@ pub fn write_forwarding_pointer<VM: VMBinding>(
 ///
 #[cfg(target_endian = "little")]
 pub(super) fn forwarding_bits_offset_in_forwarding_pointer<VM: VMBinding>() -> Option<isize> {
+    use std::ops::Deref;
     // if both forwarding bits and forwarding pointer are in-header
     match (
-        VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC,
-        VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC,
+        VM::VMObjectModel::LOCAL_FORWARDING_POINTER_SPEC.deref(),
+        VM::VMObjectModel::LOCAL_FORWARDING_BITS_SPEC.deref(),
     ) {
         (MetadataSpec::InHeader(fp), MetadataSpec::InHeader(fb)) => {
             let maybe_shift = fb.bit_offset - fp.bit_offset;
